package jve.generated;

import java.beans.*;
import java.io.InvalidObjectException;

/**
 * Default implementation of a data object that is retrieved by a service call on
 * a Data Source. 
 */
public class DataSourceDataObject extends BasicDataObject implements
		IDataSourceService {

	private boolean autoRefresh = true;

	protected boolean initialized = false;

	private Class type;

	private PropertyChangeListener argsListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			//TODO: need to be smarter and only refresh if our arg was changed
			if (isAutoRefresh())
				refresh();
		}
	};

	DataSourceService dsData = new DataSourceService(argsListener);

	/**
	 * Use the service provider to retrive an object.
	 * 
	 * Assume service/args are already set
	 * 
	 * @return service result
	 */
	protected void createObjectFromDataSouce() {
		try {
			setSourceObject(dsData.invokeDataSourceService());
			log((String) null);
		} catch (IllegalStateException e) {
			log(e);
			setSourceObject(null);
		}
	}

	/**
	 * Get the referenced object.  If the object hasn't been 
	 * initialized and the service is set up properly, the service will
	 * be queried for its value.
	 */
	public Object getObject() {
		if (!Beans.isDesignTime()) {
			if (!initialized && getSourceObject() == null)
				refresh();
			return super.getObject();
		}
		return null;
	}

	/**
	 * Refresh the value of the referenced object.  This involves 
	 * querying the service with the current values of the service args.
	 * A PropertyChangedEvent will be fired with 
	 * {@link jve.generated.IBoundObject#PROPERTY_OBJ_CHANGED PROPERTY_OBJ_CHANGED}
	 * as the property.
	 */
	public void refresh() {
		// mark ourself initialized here, as a refresh, may call as
		// object call our getObject() which may want to refresh again.
		initialized = true;

		type = null;

		if (!Beans.isDesignTime()) {
			dsData.refresh();
			createObjectFromDataSouce();
		} else
			return;

		super.refresh();

	}

	/**
	 * Set the object referenced by this data object.
	 * @param obj the referenced object
	 */
	public void setSourceObject(Object obj) {
		super.setSourceObject(obj);
		type = null;
		initialized = true;
	}

	/**
	 * Set the data source against which the service call will be performed.
	 * 
	 * @param source provider the data source
	 */
	public void setDataSource(IDataSource source) {
		dsData.setDataSource(source);
		setSourceObject(null);
		type = null;
		initialized = false;
	}

	/**
	 * Get the value of the given property from the referenced object.
	 * If the object hasn't been initialized and the service is set up 
	 * properly, the service will be queried for its value.
	 * 
	 * @param property the property value to query on the referenced object.
	 */
	public Object getValue(String property) {
		getObject();
		return super.getValue(property);
	}

	/**
	 * Get the string signature of the service method.
	 * @return method signature
	 */
	public String getServiceSignature() {
		return dsData.getServiceSignature();
	}

	/**
	 * Set the service method with the given method signature.  The
	 * arguments of the method must be included in this method signature.
	 * For example: "getEmployeeRecord(Integer id)" 
	 * 
	 * @param service the service method signature
	 */
	public void setServiceSignature(String service) {
		dsData.setServiceSignature(service);
		setSourceObject(null);
		type = null;
		initialized = false;
	}

	private IActionBinder processedAction = null;

	/**
	 * React to an action that may impact this ActionDataObject and its
	 * arguments.  This implementation will refresh this ActionDataObject
	 * and pass the action onto all of its service arguments. 
	 */
	public synchronized void actionPerformed(IActionBinder action) {
		// Avoid looping from inter-connected objects
		if (processedAction == action)
			return;

		processedAction = action;
		try {
			dsData.actionPerformed(action);
			refresh();
		} finally {
			processedAction = null;
		}
	}

	/**
	 * Set the service argument for the service call.
	 * Currently, ActionDataObjects only support service methods with a single
	 * argument.
	 * 
	 * @param arg an object reference to the service argument.
	 */
	public void setServiceArgument(final ObjectReference arg) {
		dsData.setServiceArgument(arg);
		setSourceObject(null);
		type = null;
		initialized = false;
	}

	/**
	 * Get a string representation of the ActionDataObject.
	 * @return a string representation
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();

		if (getErrorMessage() != null) {
			sb.append("***Error**: ");
			sb.append(getErrorMessage());
			sb.append("\n");
		}

		sb.append("DataSourceDataObject\n\tType: ");
		try {
			sb.append(getType());
		} catch (IllegalStateException e) {
			sb.append(e.getMessage());
		}
		sb.append("\n\tObject: ");
		sb.append(getSourceObject());
		sb.append("\n\tService:");
		sb.append(dsData.toString());
		sb.append("\n");
		return sb.toString();
	}

	/**
	 * Get an object reference to the service argument.
	 * @return the service argument
	 */
	public ObjectReference getServiceArgument() {
		return dsData.getServiceArgument();
	}

	/**
	 * Get the data source against which the service call will be performed.
	 * @return the data source
	 */
	public IDataSource getDataSource() {
		return dsData.getDataSource();
	}

	/**
	 * Get the class type of the referenced data object, which should be
	 * the return value of the service method.
	 * 
	 * @return the class type
	 * @throws InvalidObjectException if the service is not properly configured or the type can't be resolved.
	 */
	public Class getType() throws IllegalStateException {
		if (type != null)
			return type;
		try {
			type = super.getType();
		} catch (IllegalStateException e) {
			type = null;
		}
		if (type == null) {
			//  Object/Class were not set.  
			//  Find the class from the data source
			type = dsData.getType();
			if (getProperty() != null)
				type = PropertyHelper.getType(type, getProperty());
		}
		return type;
	}

	/**
	 * Get the value of the autoRefresh property.
	 * @return Returns the autoRefresh.
	 * @see #setAutoRefresh(boolean)
	 */
	public boolean isAutoRefresh() {
		return autoRefresh;
	}

	/**
	 * Sets the autoRefresh property.  If autoRefresh is enabled, the
	 * data object will refresh its value (perform the service call on the data source)
	 * whenever it receives a property change event on any of its service arguments.
	 *  
	 * @param autoRefresh true to enable autoRefresh, false otherwise.
	 */
	public void setAutoRefresh(boolean autoRefresh) {
		this.autoRefresh = autoRefresh;
	}
}