package jve.generated;

import java.beans.Beans;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;

/**
 * Default implementation of a row data object that is retrieved by a service call on
 * a Data Source. 
 */
public class DataSourceDataRows extends ObjectReference implements
		IRowsDataObject, IDataSourceService {

	protected boolean initialized = false;

	protected List listeners = new ArrayList();

	private Object previousObject = null;

	private PropertyChangeListener argsListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			refresh();
		}
	};

	DataSourceService dsData = new DataSourceService(argsListener);

	private Class type = null;

	/**
	 * Retrive the rows object by invoking the service method on the data source.
	 */
	protected void createRowsFromDataSouce() {
		try {
			setSourceObject(dsData.invokeDataSourceService());
			log((String) null);
		} catch (IllegalStateException e) {
			log(e);
			setSourceObject(null);
		}
	}

	public Object getObject() {
		if (!Beans.isDesignTime()) {
			if (!initialized && getSourceObject() == null)
				refresh();
			return super.getObject();
		}
		return null;
	}

	public void setSourceObject(Object rows) {

		if (rows != null && !(rows instanceof Object[]))
			throw new IllegalArgumentException("Object Array Expected");

		if (getProperty() != null)
			throw new IllegalArgumentException("property is set");

		super.setSourceObject(rows);
		type = null;
		initialized = false;
	}

	public void setServiceSignature(String service) {
		dsData.setServiceSignature(service);
		setSourceObject(null);
		type = null;
		initialized = false;
	}

	public String getServiceSignature() {
		return dsData.getServiceSignature();
	}

	public void setServiceArgument(ObjectReference arg) {
		dsData.setServiceArgument(arg);
		setSourceObject(null);
		type = null;
		initialized = false;
	}

	public ObjectReference getServiceArgument() {
		return dsData.getServiceArgument();
	}

	public void refresh() {
		// mark ourself initialized here, as a refresh, may call as
		// object call our getObject() which may want to refresh again.
		initialized = true;

		Object old = super.getObject();
		type = null;

		if (!Beans.isDesignTime()) {
			dsData.refresh();
			createRowsFromDataSouce();
		} else
			return;

		super.refresh();

		if (old != super.getObject()) {
			fireRowChanged(ALL_ROWS, previousObject, super.getObject());
		}
	}

	public Object get(int index) {
		Object[] rows = (Object[]) getObject();
		if (rows != null && rows.length > index)
			return rows[index];
		return null;
	}

	public void set(int index, Object value) {
		Object current = get(index);
		Object[] rows = (Object[]) getObject();
		if (rows == null || rows.length <= index)
			throw new InvalidParameterException("Row[" + index
					+ "] does not exists");
		rows[index] = value;
		fireRowChanged(index, current, value);
	}

	private IActionBinder processedAction = null;

	public synchronized void actionPerformed(IActionBinder action) {
		// Avoid looping from inter-connected objects
		if (processedAction == action)
			return;

		processedAction = action;
		try {
			dsData.actionPerformed(action);
			refresh();
		} finally {
			processedAction = null;
		}
	}

	public void setDataSource(IDataSource source) {
		dsData.setDataSource(source);
		setSourceObject(null);
		type = null;
		initialized = false;
	}

	protected void fireRowChanged(int index, Object oldVal, Object newVal) {
		RowChangeEvent event = new RowChangeEvent(this, index, oldVal, newVal);
		for (int i = 0; i < listeners.size(); i++) {
			((RowChangeListener) listeners.get(i)).rowChange(event);
		}
	}

	public void addRowChangeListener(RowChangeListener l) {
		if (!listeners.contains(l))
			listeners.add(l);
	}

	public void removeRowChangeListener(RowChangeListener l) {
		listeners.remove(l);
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (getErrorMessage() != null) {
			sb.append("***Error**: ");
			sb.append(getErrorMessage());
			sb.append("\n");
		}

		sb.append("DataSourceDataRows\n\tType: ");
		try {
			sb.append(getType());
		} catch (IllegalStateException e) {
			sb.append(e.getMessage());
		}
		sb.append("\n\tObject: ");
		sb.append(getSourceObject());
		sb.append("\n\tService:");
		sb.append(dsData.toString());
		sb.append("\n");
		return sb.toString();
	}

	public IDataSource getDataSource() {
		return dsData.getDataSource();
	}

	/**
	 * @return ComponentType of the Array
	 */
	public Class getType() throws IllegalStateException {
		if (type != null)
			return type;
		try {
			type = super.getType();
		} catch (IllegalStateException e) {
			type = null;
		}
		if (type == null) {
			//  Object/Class were not set.  
			//  Find the class from the data source
			type = dsData.getType();
		}

		if (getProperty() != null) {
			type = PropertyHelper.getType(type, getProperty());
		}
		if (type.isArray())
			type = type.getComponentType();
		return type;
	}

	/** 
	 * @return the number of rows, or -1 if the array is null
	 */
	public int getLength() {
		Object[] rows = (Object[]) getObject();
		if (rows != null)
			return rows.length;
		return -1;
	}
}