package jve.generated;

/**
 * A FieldBinder contains a String text representation of a bound property of 
 * an object pointed to by a DataObject. The bound property may be a nested 
 * property as defined by the DataObject.
 */
public interface IFieldBinder {

	/**
	 * Set the data object that provides the data used by this binder.
	 * @param object the data object referencing the target object
	 */
	public void setDataObject(IDataObject object);

	/**
	 * Get the data object that provides the data to this binder.
	 * @return the data object
	 */
	public IDataObject getDataObject();

	/**
	 * Set the property of the object that will be used as the target of
	 * this binder.  This property should have a string representation
	 * that will be used in this field.
	 * @param property the property to set.
	 */
	public void setProperty(String property);

	/**
	 * Get the property of the data object that will be used by this binder
	 * @return the property
	 */
	public String getProperty();

	// These methods will be called to map the text field value
	// into a targeted property and vice versa.
	// They may be overridden to add additional validation support
	/**
	 * Convert a string representation of a property value back to the
	 * proper object type of the property.
	 * 
	 * @param text the string of the property value.
	 * @return the Object that is represented by the string.
	 */
	public Object textToObject(String text) throws IllegalArgumentException;

	/**
	 * Convert the given object to a string representation of the value.
	 * The object should be able to be recreated from this string representation
	 * using the textToObject method.
	 * 
	 * @param o the object to convert
	 * @return the string representation of this object.
	 */
	public String objectToText(Object o);
}