package jve.generated;

/**
 * A filter binder provides a simple filter mechanism to filter objects.
 */
public interface IFilterBinder {

	/**
	 * Filter on the given object.
	 * @return true if the object is accepted by the filter (to be included), false otherwise.
	 */
	public boolean accept(Object o);

	/**
	 * A filter change event is fired when a user needs to reaccept.
	 * For instance, if the filter is bound to a text field, as the text changes
	 * in the text field, FilterChangeEvents will be fired. 
	 */
	public class FilterChangeEvent extends java.util.EventObject {

		Object newValue, oldValue;

		public FilterChangeEvent(Object source, Object oldValue, Object newValue) {
			super(source);

			this.newValue = newValue;
			this.oldValue = oldValue;
		}

		public Object getNewValue() {
			return newValue;
		}

		public Object getOldValue() {
			return oldValue;
		}
	}

	/**
	 * Interface used to register for filter change notification.
	 */
	public interface FilterChangeListener extends java.util.EventListener {

		/**
		 * filterChange is called when the value of the filter is changed.
		 * The registered listener should most likely reevaluate its filtered
		 * values based on the new filter.
		 * 
		 * @param evt the FilterChangeEvent
		 */
		void filterChange(FilterChangeEvent evt);
	}

	/**
	 * Add a new filter change listener to the listener list.
	 * @param l the new listener.
	 */
	public void addFilterChangeListener(FilterChangeListener l);

	/**
	 * Remove an existing filter change listener from the listener list. 
	 * @param l the listener to be removed.
	 */
	public void removeFilterChangeListener(FilterChangeListener l);
}