package jve.generated;

/**
 * This interface represents an array (rows) of objects.  Individual elements in the array
 * can be accessed and modified by using the get() and set() methods.  Interested
 * parties can register as a RowChangeListener to be notified when the contents of
 * a particular row are changed.  When all rows change, the RowChangeEvent will report
 * {@link #ALL_ROWS ALL_ROWS} as the changed index. 
 */
public interface IRowsDataObject extends IBoundObject {

	/** Number of example rows to generate in design time */
	public final static int DESIGNTIME_ROWS = 10;

	/** Row index used to indicate all rows changing in an RowChangeEvent */
	public final static int ALL_ROWS = -1;

	/**
	 * A RowChangeEvent is fired when the contents of a row changes in the array
	 * maintained by the RowDataObject.  If the entire contents of the array is
	 * changed or if the referenced object changes, a RowChangeEvent is fired with
	 * {@link IRowsDataObject#ALL_ROWS ALL_ROWS} as the index.
	 */
	public class RowChangeEvent extends java.util.EventObject {
		int index;

		Object newValue, oldValue;

		/** 
		 * @param source for the event
		 * @param index of row that was changed
		 * @param oldValue
		 * @param newValue
		 */
		public RowChangeEvent(Object source, int index, Object oldValue,
				Object newValue) {
			super(source);
			this.index = index;
			this.newValue = newValue;
			this.oldValue = oldValue;
		}

		public int getIndex() {
			return index;
		}

		public Object getNewValue() {
			return newValue;
		}

		public Object getOldValue() {
			return oldValue;
		}
	}

	/**
	 * Interface used to register for row change notification. 
	 */
	public interface RowChangeListener extends java.util.EventListener {
		/**
		 * rowChange is called when the value of a particular row is changed, or
		 * if the value of all rows are changed.
		 * 
		 * @param evt the RowChangeEvent
		 */
		void rowChange(RowChangeEvent evt);
	}

	/**
	 * Retrieve the value of the row at the given index.
	 * @param index index of the row to retrieve 
	 */
	public Object get(int index);

	/**
	 * Set the value at the given index
	 * @param index the index to set
	 * @param value the new value
	 */
	public void set(int index, Object value);

	/**
	 * Add a new row change listener to the listener list.
	 * @param l the new listener.
	 */
	public void addRowChangeListener(RowChangeListener l);

	/**
	 * Remove an existing row change listener from the listener list. 
	 * @param l the listener to be removed.
	 */
	public void removeRowChangeListener(RowChangeListener l);

	/**
	 * Returns the number of rows in this array
	 * @return array length, or -1 if the value of the array is null.
	 */
	public int getLength();
}