package jve.generated;

import java.util.EventListener;

/**
 * 
 * A Table binder represets an array of objects (rows) that are the basis for
 * a table model.  This model will be bound to a service or another 
 * object.
 * <p>
 * A Table binder also exports an Object (from the IDataObject interface)
 * that represents the selected object (row[selectedIndex]) in the visual table. Because
 * of this, a ITableBinder can only be associated with a single visual table.
 * <p>
 * If the visual table is setup to allow multiple row selections, the selection object
 * will be the first selected row.
 */
public interface ITableBinder extends IDataObject {

	/**
	 * Get the value of the given cell in the table.
	 * 
	 * @param rowIndex  the cell's row
	 * @param columnIndex the cell's column
	 * @return the Object in that cell
	 */
	public Object getValueAt(int rowIndex, int columnIndex);

	/**
	 * Get the value of the given property from the object at the given row (index in
	 * the array of objects.)
	 * @param rowIndex the index of the object in the array
	 * @param property the property of the object
	 * @return the value of the property on the object in the row
	 */
	public Object getValueAt(int rowIndex, String property);

	/**
	 * Get the name of the given column.  This text is usually displayed on a
	 * table column header.
	 * 
	 * @param column the column index
	 * @return the name of the column
	 */
	public String getColumnName(int column);

	/**
	 * Get the number of rows in the table.  This number reflects the number
	 * of rows after filtering, if filtering is enabled.
	 * 
	 * @return number of rows
	 */
	public int getRowCount();

	/**
	 * Get the number of columns available for the table.  If the
	 * binder is initialized properly, this will reflect the number
	 * of properties available on the row object type.
	 * 
	 * @return the number of available rows.
	 */
	public int getColumnCount();

	/**
	 * Notice of a change of selection in the ITableBinder's visual table.
	 * The new selected object can be retrieved from the source's <code>getObject()</code>
	 * method. 
	 */
	public interface SelectionChangedEvent {
		/**
		 * Get the source ITableBinder (may cast).
		 * @return the source ITableBinder
		 */
		public Object getSource();

		/**
		 * This returns the index of the selected row in the visual table.
		 * @return the index, or -1 if no selection.
		 */
		public int getSelectionIndex();
	}

	/**
	 * Listen for changes to the ITableBinder's selected object. 
	 *
	 */
	public interface SelectionChangedListener extends EventListener {

		/**
		 * Callback notification for selected object changes
		 * @param e the SelectionChangedEvent.
		 */
		void selectionChanged(SelectionChangedEvent e);
	}

	/**
	 * This returns the index of the selected row in the visual table.
	 * @return the index, or -1 if no selection.
	 */
	public int getSelectedIndex();

	/**
	 * Get the selected object for the currently selected row in the visual table.
	 * @return the selected object, null if there's no selection.
	 */
	public Object getSelectedObject();

	/**
	 * Add a listener for table selection changes
	 * @param l the SelectionChangedListener
	 */
	public void addSelectionChangedListener(SelectionChangedListener l);

	/**
	 * Remove an existing table selection listener
	 * @param l the listener to remove
	 */
	public void removeSelectionChangedListener(SelectionChangedListener l);

	/**
	 * Set a filter to attatch to this table.  This is optional.
	 * @param filter the filter to use with this table.
	 */
	public void setFilter(IFilterBinder filter);

	/**
	 * Get the filter used by this table.
	 * @return the filter, null if none is set.
	 */
	public IFilterBinder getFilter();

	/**
	 * Get the index of the row data associated with this table binder 
	 * that corresponds to the given index in the visual table.  
	 * These values may not be equal due to filtering and sorting 
	 * of the rows on the visual table.
	 * 
	 * @param index the row index from the visual table
	 * @return the corresponding row index from the row data
	 */
	public int getDataIndexFromVisualIndex(int index);

	/**
	 * Get the index of the visual table row that corresponds to the
	 * given index in the row data associated with this table binder.
	 * These values may not be equal due to filtering and sorting 
	 * of the rows on the visual table.
	 * 
	 * @param index the row index from the row data
	 * @return the corresponding row index from the visual table
	 */
	public int getVisualIndexFromDataIndex(int index);
}