package jve.generated;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.BeanInfo;
import java.beans.Beans;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyDescriptor;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import jve.generated.IFilterBinder.FilterChangeEvent;
import jve.generated.IFilterBinder.FilterChangeListener;

/**
 * Swing implementation of the ITableBinder interface.  This class acts as a model for a Swing JTable.
 * It allows the sorting and filtering of JTable's contents.  The SwingTableBinder also exports the selected
 * row's element as a DataObject.<p>
 * The SwingTableBinder cannot be used directly, rather its subclasses 
 * {@link jve.generated.JRowTableBinder JRowTableBinder} or {@link jve.generated.JObjectTableBinder JObjectTableBinder} 
 * should be used depending on the source of array data the table is bound to.
 */
public abstract class SwingTableBinder extends AbstractTableModel implements
		ITableBinder {

	protected JTable visualTable = null; // Visual Table

	protected IFilterBinder tableFilter = null;

	protected Object[] modelRows = null; // This is the sorted/filtered version of the data

	protected Object[] unSortedModelRows = null;

	protected HashMap modelToSourceIndex = new HashMap(); // Map to the original index

	protected HashMap sourceToModelIndex = new HashMap(); // Map to visual Index

	protected HashMap rowToValue = new HashMap(); // Cache the property helper for a given row/property

	protected String[] modelHeaders = null; // DEFAULT col. headers

	// The following will not be exposed on the property sheet
	protected List modelProperties = null; // Properties available for the model

	// listed on the table
	protected List listeners = new ArrayList();

	protected IDataObject selectedRowDataObject = new BasicDataObject();

	private String fdebugMsg = null;

	private TableCellRenderer orgTableCellRenderer;

	// This listener is responsible to maintain the mapping between a Column's
	// "identifier" property and the a column's model index
	TableColumnModelListener identifierListener = new TableColumnModelListener() {
		PropertyChangeListener pcl = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if ("identifier".equals(evt.getPropertyName())) {
					TableColumn tc = (TableColumn) evt.getSource();
					updateColumnModelIndex(tc);
				}
			}
		};

		public void columnMarginChanged(ChangeEvent e) {
		}

		public void columnSelectionChanged(ListSelectionEvent e) {
		}

		public void columnAdded(TableColumnModelEvent e) {
			TableColumnModel model = (TableColumnModel) e.getSource();
			TableColumn tc = model.getColumn(e.getToIndex());
			updateColumnModelIndex(tc);
			tc.addPropertyChangeListener(pcl);
		}

		public void columnMoved(TableColumnModelEvent e) {
		}

		public void columnRemoved(TableColumnModelEvent e) {
			// Col. is already removed
		}
	};

	/**
	 * The type of event that is fired when the table's selection changes.
	 */
	public class SwingTableSelectionChangedEvent extends EventObject implements
			ITableBinder.SelectionChangedEvent {
		public SwingTableSelectionChangedEvent(Object source) {
			super(source);
		}

		public int getSelectionIndex() {
			return getSelectedIndex();
		}
	}

	private SwingTableSelectionChangedEvent selectionChangeEvent = new SwingTableSelectionChangedEvent(
			this);

	// The Table's selection listener will route the event to the binder's
	// listeners
	ListSelectionListener visualTableSelectionListener = new ListSelectionListener() {

		public void valueChanged(final ListSelectionEvent e) {
			if (e.getValueIsAdjusting())
				return;

			fireSelectionChanged(selectionChangeEvent);
		}
	};

	FilterChangeListener filterListener = new FilterChangeListener() {
		public void filterChange(FilterChangeEvent evt) {
			reset(false);
		}
	};

	//	 Listern for clicks on the table's header to drive the sorting of colums
	private MouseListener mouseListener = new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
			JTableHeader h = (JTableHeader) e.getSource();
			TableColumnModel columnModel = h.getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			int columnModelIndex = columnModel.getColumn(viewColumn)
					.getModelIndex();
			if (columnModelIndex != -1) {
				int status = columnModelIndex == sortedColumnModelIndex ? sortingStatus
						: NOT_SORTED;
				if (!e.isControlDown()) {
					visualTable.getTableHeader().repaint();
				}
				status = status + (e.isShiftDown() ? -1 : 1);
				status = (status + 1) % 3 - 1; // signed mod returning {-1, 0, 1}
				sortedColumnModelIndex = columnModelIndex;
				sortingStatus = status;
				Object selected = getSelectedObject();
				performSort();
				fireTableDataChanged();
				if (selected != null) {
					for (int i = 0; i < getRows().length; i++) {
						if (selected == getRows()[i]) {
							visualTable.getSelectionModel()
									.setSelectionInterval(i, i);
							break;
						}
					}
				}
			}
		}
	};

	// Simple sorting support - single column using toString()
	public static final int NOT_SORTED = 0;

	public static final int DESCENDING = -1;

	public static final int ASCENDING = 1;

	int sortingStatus = NOT_SORTED;

	int sortedColumnModelIndex = -1;

	// Locale specific comparator
	Collator localeComparitor = Collator.getInstance(Locale.getDefault());

	/**
	 * The log(xxx) methods are called in case of a failure.  
	 * Data Objects and binders do not intend to provide any error 
	 * validation framework. The default logging support will 
	 * use the error stream if the -Djve.debug system property
	 * is defined.
	 * 
	 * One can override these methods to hook up to a particular error 
	 * validation/framework
	 *
	 * @param t Exceptions
	 */
	protected void log(Throwable t) {
		if (ObjectReference.debug && t != null) {
			t.printStackTrace();
		}
		if (t != null)
			fdebugMsg = t.getMessage();
		else
			fdebugMsg = null;
	}

	protected void log(String s) {
		if (ObjectReference.debug && s != null)
			System.err.println(s);
		fdebugMsg = s;
	}

	protected String getSortedProperty() {
		String sortedProperty = null;
		if (sortedColumnModelIndex >= 0)
			sortedProperty = (String) modelProperties
					.get(sortedColumnModelIndex);
		return sortedProperty;
	}

	public Comparator rowSortComparitor = new Comparator() {
		public int compare(Object o1, Object o2) {
			o1 = primGetValue(o1, getSortedProperty());
			o2 = primGetValue(o2, getSortedProperty());
			if (o1 == null) {
				if (o2 == null)
					return 0;
				return 1;
			} else if (o2 == null)
				return -1;

			int result = 0;
			if (o1 instanceof Comparable) {
				// If the objects being compared implement comparable, use the comparable compare
				result = ((Comparable) o1).compareTo(o2);
			} else {
				// Otherwise compare based on the toString values of the objects
				result = localeComparitor.compare(o1.toString(), o2.toString());
			}
			if (sortingStatus == DESCENDING)
				return -result;
			return result;
		}
	};

	/**
	 * Sort the rows of the table.
	 *
	 */
	protected void performSort() {
		if (modelRows == null || getRowCount() < 2)
			return;

		sourceToModelIndex.clear();

		if (sortingStatus != NOT_SORTED)
			Arrays.sort(modelRows, rowSortComparitor);
		else
			modelRows = (Object[]) unSortedModelRows.clone();
	}

	/*
	 * An arrow icon placed on the table column header to indicate the sorting status.
	 */
	private static class SortArrow implements Icon {
		private boolean descending;

		private int size;

		public SortArrow(boolean descending, int size) {
			this.descending = descending;
			this.size = size;
		}

		public void paintIcon(Component c, Graphics g, int x, int y) {
			Color color = c == null ? Color.gray : c.getBackground();
			int dx = (int) (size / 1.5);
			int dy = descending ? dx : -dx;
			y = y + 5 * size / 6 + (descending ? -dy : 0);
			int shift = descending ? 1 : -1;
			g.translate(x, y);
			// Right diagonal.
			g.setColor(color.darker());
			g.drawLine(dx / 2, dy, 0, 0);
			g.drawLine(dx / 2, dy + shift, 0, shift);
			// Left diagonal.
			g.setColor(color.brighter());
			g.drawLine(dx / 2, dy, dx, 0);
			g.drawLine(dx / 2, dy + shift, dx, shift);
			// Horizontal line.
			if (descending) {
				g.setColor(color.darker().darker());
			} else {
				g.setColor(color.brighter().brighter());
			}
			g.drawLine(dx, 0, 0, 0);
			g.setColor(color);
			g.translate(-x, -y);
		}

		public int getIconWidth() {
			return size;
		}

		public int getIconHeight() {
			return size;
		}
	}

	/**
	 * Get the sorting icon for the given column.
	 */
	protected Icon getHeaderRendererIcon(int column, int size) {
		if (sortingStatus == NOT_SORTED || column != sortedColumnModelIndex)
			return null;

		return new SortArrow(sortingStatus == DESCENDING, size);
	}

	private class SortHeadRenderer implements TableCellRenderer {
		private TableCellRenderer tableCellRenderer;

		public SortHeadRenderer(TableCellRenderer tableCellRenderer) {
			this.tableCellRenderer = tableCellRenderer;
		}

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Component c = tableCellRenderer.getTableCellRendererComponent(
					table, value, isSelected, hasFocus, row, column);
			if (c instanceof JLabel) {
				JLabel l = (JLabel) c;
				l.setHorizontalTextPosition(JLabel.LEFT);
				int modelColumn = table.convertColumnIndexToModel(column);
				l.setIcon(getHeaderRendererIcon(modelColumn, l.getFont()
						.getSize()));
			}
			return c;
		}
	}

	/**
	 * Notify all SelectionChangedListeners that the selection on the table has changed.
	 * @param e event detailing the change in selection
	 */
	protected void fireSelectionChanged(ITableBinder.SelectionChangedEvent e) {
		selectedRowDataObject.setSourceObject(getSelectedObject());
		for (int i = 0; i < listeners.size(); i++) {
			((ITableBinder.SelectionChangedListener) listeners.get(i))
					.selectionChanged(e);
		}
	}

	/**
	 * Filter the rows of the table if a filter is set on this binder.
	 *
	 */
	protected void performFilter() {
		if (tableFilter != null && modelRows != null && !Beans.isDesignTime()) {
			sourceToModelIndex.clear();
			ArrayList list = new ArrayList();
			for (int i = 0; i < modelRows.length; i++) {
				if (tableFilter.accept(modelRows[i]))
					list.add(modelRows[i]);
			}
			modelRows = list.toArray();
		}
	}

	/**
	 * Get the object this table is bound to.
	 * @return the source {@link jve.generated.IBoundObject IBoundObject} for the table array.
	 */
	protected abstract IBoundObject getBoundObject();

	/**
	 * Get the data rows from the bound object.
	 * @return array of data provided by the bound object.
	 */
	protected abstract Object[] primGetRows();

	/**
	 * Clear out all cached information stored in the binder.
	 *
	 */
	protected void clearModel() {
		modelRows = null;
		unSortedModelRows = null;
		modelProperties = null;
		modelHeaders = null;
		modelToSourceIndex.clear();
		sourceToModelIndex.clear();
		rowToValue.clear();
	}

	/**
	 * Cache information regarding the table rows.
	 * @throws IntrospectionException if introspection fails on the data rows' type.
	 */
	protected void updateRowInformation() throws IntrospectionException {

		// Update available properties if needed
		if (modelProperties == null) {
			Class rtype = null;
			rtype = getBoundObject().getType();
			if (rtype.isArray())
				rtype = rtype.getComponentType();
			BeanInfo bi = Introspector.getBeanInfo(rtype);
			// Get properties that have get methods on them
			PropertyDescriptor[] pds = bi.getPropertyDescriptors();
			modelProperties = new ArrayList();
			for (int i = 0; i < pds.length; i++) {
				if (pds[i].getReadMethod() != null && !pds[i].isHidden())
					modelProperties.add(pds[i].getName());
			}
			// New properties were set... reflect this on the
			// table columns.
			updateAllTableColumnIndexes();
		}
	}

	/**
	 * Get the row data to display in the table.  This list will be filtered and sorted if
	 * necessary.
	 * 
	 * @return an array of row elements
	 */
	protected Object[] getRows() {
		if (modelRows != null)
			return modelRows;

		// No cached rows ... 
		if (getBoundObject() == null) {
			clearModel();
		} else {
			modelRows = primGetRows();
			try {
				updateRowInformation();
				log((String) null);
			} catch (IntrospectionException e) {
				log(e);
			}
			if (modelRows != null) {
				performFilter();
				unSortedModelRows = (Object[]) modelRows.clone();
				performSort();
				fireTableDataChanged();
			}
		}
		return modelRows;
	}

	public void addSelectionChangedListener(SelectionChangedListener l) {
		if (!listeners.contains(l))
			listeners.add(l);
	}

	public void removeSelectionChangedListener(SelectionChangedListener l) {
		listeners.remove(l);
	}

	public int getColumnCount() {
		// if count > 0, and no columns are set on a table,
		// JTable will not be rendered on target VM.
		// If autoGenerate table from model is off (green Thread)
		// this method will never be called.
		//		return 0;
		int tcol = visualTable.getColumnModel().getColumnCount();
		if (modelRows == null || modelRows.length == 0)
			getRows();
		return modelProperties == null ? tcol : modelProperties.size();
	}

	/**
	 * Update the modelIndex propert on the table's columns.
	 * Swing's JTable is based on the modelIndex property to drive column content.
	 * This binder will force the index to match the Z-order of the property
	 * the column is associated with the "identifier" property.
	 */
	protected void updateColumnModelIndex(TableColumn tc) {
		if (tc != null) {
			if (tc.getIdentifier() != null && modelProperties != null) {
				String prop = tc.getIdentifier().toString();
				for (int j = 0; j < modelProperties.size(); j++) {
					if (modelProperties.get(j).equals(prop)) {
						if (tc.getModelIndex() != j) {
							tc.setModelIndex(j);
							fireTableStructureChanged();
						}
						break;
					}
				}
			}
		}
	}

	/**
	 * Update the table column indexes to match the Z-order of the properties used
	 * to define the columns of the table.
	 *
	 */
	protected void updateAllTableColumnIndexes() {
		if (visualTable != null && modelProperties != null) {
			for (int i = 0; i < visualTable.getColumnCount(); i++)
				updateColumnModelIndex(visualTable.getColumnModel()
						.getColumn(i));
		}
	}

	public int getRowCount() {
		int result = getRows() == null ? 0 : getRows().length;
		if (Beans.isDesignTime()) {
			if (modelProperties != null)
				result = IRowsDataObject.DESIGNTIME_ROWS;
		}
		return result;
	}

	public int getSelectedIndex() {
		return visualTable.getSelectedRow();
	}

	public Object getSelectedObject() {
		if (getRows() != null && getSelectedIndex() >= 0)
			return getRows()[getSelectedIndex()];
		return null;
	}

	/**
	 * Get the value of the cell specified by the given row and column property using a cached
	 * PropertyHelper.
	 * @param row the row index
	 * @param property the property of the object at the target row
	 * @return value of the given property
	 */
	private Object primGetValue(Object row, String property) {
		if (row == null)
			return null;
		HashMap values = (HashMap) rowToValue.get(row);
		if (values == null) {
			values = new HashMap();
			rowToValue.put(row, values);
		}
		PropertyHelper helper = (PropertyHelper) values.get(property);
		if (helper == null) {
			helper = new PropertyHelper(row, property.toString());
			values.put(property, helper);
		}
		return helper.getValue();
	}

	/**
	 * Get the value of the cell at the given row and column index.
	 * This will return error strings if the table binder is not properly setup.
	 * @param rowIndex the row index
	 * @param columnIndex the column index
	 * @return the value of the cell
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		try {
			Object property = null;
			// Find the (first) column noting this index, and get its property
			for (int i = 0; i < visualTable.getColumnCount(); i++)
				if (visualTable.getColumnModel().getColumn(i).getModelIndex() == columnIndex)
					property = visualTable.getColumnModel().getColumn(i)
							.getIdentifier();

			if (property == null) {
				return "No Identifier Property defined";
			}

			int dotIndex = property.toString().indexOf('.');
			if ((dotIndex == -1) ? !modelProperties.contains(property)
					: !modelProperties.contains(property.toString().substring(
							0, dotIndex))) {
				return "No Identifier Property defined";
			}

			if (Beans.isDesignTime()) {
				return "[" + property + " " + rowIndex + "]";
			}

			// get the proper property of this JavaBean
			Object row = getRows()[rowIndex];
			if (columnIndex >= modelProperties.size())
				return "Error on Column: " + columnIndex;

			Object result = primGetValue(row, property.toString());
			log((String) null);
			return result;
		} catch (Exception e) {
			log(e);
		}
		return null;
	}

	/**
	 * Get the value of the cell specified by the given row and column property
	 * @param rowIndex the row index
	 * @param property the property of the object at the target row
	 * @return value of the given property
	 */
	public Object getValueAt(int rowIndex, String property) {
		try {
			if (modelProperties != null && modelProperties.contains(property)) {
				Object row = getRows()[rowIndex];
				Object result = primGetValue(row, property);
				log((String) null);
				return result;
			}
		} catch (Exception e) {
			log(e);
		}
		return null;
	}

	/**
	 * Get the default name of the column at the given index.  This will be the
	 * row object property set for the given column.
	 * 
	 * @param column the index of the column to get
	 * @return the name of the column.
	 */
	public String getColumnName(int column) {
		if (column >= getColumnCount())
			return ("BadCol[" + column + "]");
		// Set up the table's headers
		if (modelHeaders == null) {
			modelHeaders = (String[]) modelProperties
					.toArray(new String[modelProperties.size()]);
		}
		return modelHeaders[column];
	}

	/**
	 * Hook up the table binder to work with the given visual table.  This table binder should be
	 * the model for the target visual table.
	 * 
	 * @param table the table that uses this table binder.
	 */
	protected void setVisualTable(JTable table) {
		if (visualTable != null) {
			visualTable.getSelectionModel().removeListSelectionListener(
					visualTableSelectionListener);
			visualTable.getTableHeader().removeMouseListener(mouseListener);
			// We should restore the Header's cell renderer
			visualTable.getTableHeader().setDefaultRenderer(
					orgTableCellRenderer);
			visualTable.getColumnModel().removeColumnModelListener(
					identifierListener);
		}
		visualTable = table;
		if (table == null)
			return;
		visualTable.getSelectionModel().addListSelectionListener(
				visualTableSelectionListener);
		visualTable.getTableHeader().addMouseListener(mouseListener);
		// Store the original header's cell renderer so we can restore it later if need be.
		orgTableCellRenderer = visualTable.getTableHeader()
				.getDefaultRenderer();
		visualTable.getTableHeader().setDefaultRenderer(
				new SortHeadRenderer(visualTable.getTableHeader()
						.getDefaultRenderer()));

		visualTable.getColumnModel().addColumnModelListener(identifierListener);
		updateAllTableColumnIndexes();
	}

	/**
	 * Add a table model listener to the listener list.  We override this method to automaticaly setup
	 * this binder to use the target JTable.
	 * @param l the listener to add
	 */
	public void addTableModelListener(TableModelListener l) {
		super.addTableModelListener(l);
		if (l instanceof JTable)
			setVisualTable((JTable) l);
	}

	/**
	 * Remove a table model listener from the list.  If the target listener is the JTable associated with
	 * this binder, the binder unhooks itself from the table.
	 */
	public void removeTableModelListener(TableModelListener l) {
		super.removeTableModelListener(l);
		if (l == visualTable) {
			setVisualTable(null);
		}
	}

	/**
	 * Refresh model data, and re-drive the current selection.
	 */
	public void refresh() {
		Object selected = getSelectedObject();
		int selectedIndex = getSelectedIndex();
		clearModel();
		getBoundObject().refresh();
		Object[] rows = getRows();

		if (rows != null && rows.length > 0 && selectedIndex >= 0) {
			//	First try to find the old object
			for (int i = 0; i < rows.length; i++) {
				if (rows[i].equals(selected)) {
					// found an actuall match
					visualTable.getSelectionModel().setSelectionInterval(i, i);
					return;
				}
			}
			// select the same row
			if (selectedIndex >= rows.length)
				selectedIndex = rows.length - 1;
			visualTable.getSelectionModel().setSelectionInterval(selectedIndex,
					selectedIndex);
		}
	}

	private IActionBinder processedAction = null;

	/**
	 * Respond to an action occuring.  This implementation will refresh the table's contents.
	 */
	public synchronized void actionPerformed(IActionBinder action) {
		// Avoid looping from inter-connected objects
		if (processedAction == action)
			return;

		processedAction = action;
		try {
			refresh();
		} finally {
			processedAction = null;
		}
	}

	/**
	 * Add a PropertyChangeListener to the selected data object.
	 */
	public void addPropertyChangeListener(PropertyChangeListener l) {
		selectedRowDataObject.addPropertyChangeListener(l);
	}

	/**
	 * Remove a PropertyChangeListener from the selected data object.
	 */
	public void removePropertyChangeListener(PropertyChangeListener l) {
		selectedRowDataObject.removePropertyChangeListener(l);
	}

	/**
	 * Get the selected row in the visual table.
	 * 
	 * @return the selected row object, null if there is no selection in the table.
	 */
	public Object getObject() {
		return selectedRowDataObject.getObject();
	}

	public Object getSourceObject() {
		return getObject();
	}

	/**
	 * Get the value of the given property from the row object that is selected
	 * in the visual table.
	 * 
	 * @param property the property of the row object
	 * @return the value, null if there is no selection in the table.
	 */
	public Object getValue(String property) {
		return selectedRowDataObject.getValue(property);
	}

	/**
	 * This method should not be called on a SwingTableBinder.  Use the visual table
	 * to change the selected object on the table.
	 */
	public void setSourceObject(Object obj) {
		throw new IllegalStateException("Cannot override selected object.");
	}

	/**
	 * This method should not be called on a SwingTableBinder.
	 */
	public void setSourceObject(Object obj, String property) {
		throw new IllegalStateException("Cannot override selected object.");
	}

	/**
	 * This method should not be called on a SwingTableBinder.
	 */
	public void setProperty(String property) {
		throw new IllegalStateException("Cannot override selected object.");
	}

	/**
	 * There is no bound property on a SwingTableBinder.
	 * @return this will always return null
	 */
	public String getProperty() {
		return null;
	}

	/**
	 * Change the value of the given property on the selected row object in the table.
	 * @param property the target property of the row object
	 * @param value the value to set
	 */
	public void setValue(String property, Object value) {
		selectedRowDataObject.setValue(property, value);
		fireTableRowsUpdated(getSelectedIndex(), getSelectedIndex());
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (fdebugMsg != null) {
			sb.append("***Error**: ");
			sb.append(fdebugMsg);
			sb.append("\n");
		}
		sb.append("TableBinder\n\t");
		sb.append(getBoundObject().toString());
		sb.append("\n\tVisualTable:\n\t");
		sb.append(visualTable);
		sb.append("\n");
		return sb.toString();
	}

	/**
	 * Respond to the change of the table model (such as changing target rows data source). This will
	 * clear out any cached table information and update the visual table.
	 * 
	 * @param hard if true the assumption is that the source rows object has changed
	 *             else, sorting, or filtering has changed.
	 */
	protected void reset(boolean hard) {
		modelRows = null;
		if (hard) {
			clearModel();
		}
		fireTableStructureChanged();
	}

	public IFilterBinder getFilter() {
		return tableFilter;
	}

	public void setFilter(IFilterBinder filter) {
		if (tableFilter != null) {
			tableFilter.removeFilterChangeListener(filterListener);
		}
		tableFilter = filter;
		if (tableFilter != null) {
			tableFilter.addFilterChangeListener(filterListener);
		}
		reset(false);
	}

	public Class getType() {
		if (getBoundObject() != null)
			return getBoundObject().getType();
		return null;
	}

	public int getDataIndexFromVisualIndex(int index) {

		if (index < 0 || getRows() == null || getRows().length <= index)
			return -1;
		// lazy initialization
		if (modelToSourceIndex.isEmpty()) {
			Object[] sourceRows = primGetRows();
			for (int i = 0; i < sourceRows.length; i++) {
				modelToSourceIndex.put(sourceRows[i], new Integer(i));
			}
		}
		Object o = getRows()[index];
		return ((Integer) modelToSourceIndex.get(o)).intValue();
	}

	public int getVisualIndexFromDataIndex(int index) {
		if (index < 0 || getRows() == null || primGetRows().length <= index)
			return -1;

		// lazy initialization
		if (sourceToModelIndex.isEmpty()) {
			Object[] rows = getRows();
			for (int i = 0; i < rows.length; i++) {
				sourceToModelIndex.put(rows[i], new Integer(i));
			}
		}
		Object o = primGetRows()[index];
		Integer idx = (Integer) sourceToModelIndex.get(o);
		if (idx != null)
			return idx.intValue();
		return -1;
	}

	public boolean isReadOnly(String property) {
		return selectedRowDataObject.isReadOnly(property);
	}

	public boolean isPrimitive(String property) {
		return selectedRowDataObject.isPrimitive(property);
	}
}