package jve.generated;

import java.awt.Toolkit;
import java.beans.Beans;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainDocument;
import javax.swing.text.SimpleAttributeSet;

/**
 * Swing implementation of the IFieldBinder interface.  This class acts as a document for a Swing JTextComponent.
 * This field provides basic validation of the text entered into the field, to ensure the value can be parsed
 * into the bound data type.  To modify the default validation behavior, override the textToObject() method.
 */
public class SwingTextComponentBinder extends PlainDocument implements
		IFieldBinder, PropertyChangeListener {

	IDataObject dataObject = null;

	String property = null;

	boolean originalEditable;

	boolean autoEditable = true; // Should the binder control the Editable property of the text component

	JTextComponent visualTextComponent = null; // Need this to enable/disable

	private boolean updateInProgress = false; // ignore notifications during updates

	private String lastValidText = null;

	private String fdebugMsg = null;

	public SwingTextComponentBinder() {
		super();
		addDocumentListener(new DocumentListener() {
			public void changedUpdate(DocumentEvent e) {
				contentChanged(e);
			}

			public void insertUpdate(DocumentEvent e) {
				contentChanged(e);
			}

			public void removeUpdate(DocumentEvent e) {
				contentChanged(e);
			}
		});
	}

	/**
	 * @return the data object this binder is associated with.
	 */
	public IDataObject getDataObject() {
		return dataObject;
	}

	/**
	 * @return the property this field is bound to.
	 */
	public String getProperty() {
		return property;
	}

	/**
	 * @param binder
	 */
	public void setDataObject(IDataObject binder) {
		if (dataObject != null)
			dataObject.removePropertyChangeListener(this);
		dataObject = binder;
		if (dataObject != null)
			dataObject.addPropertyChangeListener(this);
		updateContent();
	}

	/**
	 * @param string
	 */
	public void setProperty(String string) {
		property = string;
		updateContent();
	}

	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String p = evt.getPropertyName();
		if (p.equals(IBoundObject.PROPERTY_OBJ_CHANGED)) {
			// Object had changed
			updateContent();
		} else if (p.equals(property)) {
			// Skip the reflection, and use the value
			updateContent(evt.getNewValue());
		}
	}

	protected boolean isNullEditable() {
		if (dataObject != null && dataObject.getSourceObject() != null) {
			if (property != null) {
				int last = property.lastIndexOf('.');
				if (last >= 0) {
					String parent = property.substring(0, last);
					PropertyHelper ph = new PropertyHelper(dataObject, parent);
					return ph.getValue() != null;
				}
				return !dataObject.isReadOnly(property);
			}
			return true;
		}
		return false;
	}

	protected void updateContent(Object o) {
		try {
			String val;
			if (o == null) {
				val = "";
				if (visualTextComponent != null && isAutoEditable())
					visualTextComponent.setEditable(isNullEditable());
			} else {
				val = objectToText(o);
				if (visualTextComponent != null && isAutoEditable())
					visualTextComponent.setEditable(!isReadOnly());
			}
			// Eliminate Feedback
			try {
				updateInProgress = true;
				remove(0, getLength());
				insertString(0, val, new SimpleAttributeSet());
				fdebugMsg = null;
			} finally {
				updateInProgress = false;
			}
		} catch (BadLocationException e) {
			fdebugMsg = e.getMessage();
		}
	}

	protected boolean isReadOnly() {
		boolean ro = true;
		try {
			ro = dataObject.isReadOnly(property);
		} catch (RuntimeException e) {
		}
		return ro;
	}

	protected void updateContent() {
		if (!Beans.isDesignTime() && dataObject != null) {
			Object o;
			if (property != null)
				o = dataObject.getValue(property);
			else
				o = dataObject.getObject();
			updateContent(o);
			try {
				lastValidText = getContent().getString(0, getLength());
			} catch (BadLocationException e) {
				fdebugMsg = e.getMessage();
			}
		}
	}

	//	protected String getOriginalValue (DocumentEvent de) {
	//	    String val = visualTextComponent.getText();
	//	    if (de.getType() == DocumentEvent.EventType.CHANGE)
	//	        val.substring(0,de.getOffset()) + de.
	//	}
	/**
	 * Drive the text field change back into the Object we are 
	 * bounded to.
	 */
	protected void contentChanged(DocumentEvent de) {
		if (updateInProgress)
			return;

		try {
			if (dataObject != null) {
				Object o = null;
				try {
					String newContent = getContent().getString(0, getLength());
					o = textToObject(newContent);
					// Eliminate feedback
					dataObject.removePropertyChangeListener(this);
					if (property != null)
						dataObject.setValue(property, o);
					else
						dataObject.setSourceObject(o);
					dataObject.addPropertyChangeListener(this);
					lastValidText = newContent;
				} catch (IllegalArgumentException e) {
					// Invalid text content
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							updateContent(lastValidText);
							Toolkit.getDefaultToolkit().beep();
						}
					});
				}
			}
		} catch (BadLocationException e) {
			fdebugMsg = e.getMessage();
		}
		fdebugMsg = null;
	}

	/**
	 * @return Returns the visualTextComponent.
	 */
	public JTextComponent getVisualTextComponent() {
		return visualTextComponent;
	}

	/**
	 * @param visualTextComponent fooa The visualTextComponent to set.
	 */
	public void setVisualTextComponent(JTextComponent visualTextComponent) {
		this.visualTextComponent = visualTextComponent;
		if (visualTextComponent != null) {
			originalEditable = visualTextComponent.isEditable();
		}
	}

	/** 
	 * Convert the given object into a text representation.
	 * Overide for special conversion semantics. 
	 * @param o the object to convert
	 * @return an text representation of the object.
	 */
	public String objectToText(Object o) {
		return o.toString();
	}

	/**
	 * Convert the given text to a Object of the type that's the bound property.
	 * Overide for special conversion semantics.
	 * @param text the text value to convert
	 * @return the converted object of the correct type
	 * @throws IllegalArgumentException if the text cannot be converted into the proper type.
	 */
	public Object textToObject(String text) throws IllegalArgumentException {
		Object result = text;
		// The following are transient characters that will be allowed on a number based field
		String[] validIntegerCharacters = new String[] { "-", "+", "" };
		String[] validFloatCharecters = new String[] { "-", "+", ".", "" };

		Class c = dataObject.getType();
		if (c != null && getProperty() != null)
			c = PropertyHelper.getType(c, getProperty());

		if (c == String.class) {
			// Most likely case
			return text;
		}
		if (c == Integer.class || c == Integer.TYPE) {
			try {
				result = new Integer(Integer.parseInt(text));
			} catch (NumberFormatException e1) {
				for (int i = 0; i < validIntegerCharacters.length; i++) {
					if (validIntegerCharacters[i].equals(text)) {
						result = new Integer(0);
						return result;
					}
				}
				throw new IllegalArgumentException(text);
			}
		} else if (c == Long.class || c == Long.TYPE) {
			try {
				result = new Long(Long.parseLong(text));
			} catch (NumberFormatException e1) {
				for (int i = 0; i < validIntegerCharacters.length; i++) {
					if (validIntegerCharacters[i].equals(text)) {
						result = new Long(0);
						return result;
					}
				}
				throw new IllegalArgumentException(text);
			}
		} else if (c == Short.class || c == Short.TYPE) {
			try {
				result = new Short(Short.parseShort(text));
			} catch (NumberFormatException e1) {
				for (int i = 0; i < validIntegerCharacters.length; i++) {
					if (validIntegerCharacters[i].equals(text)) {
						result = new Short((short) 0);
						return result;
					}
				}
				throw new IllegalArgumentException(text);
			}
		} else if (c == Double.class || c == Double.TYPE) {
			try {
				result = new Double(Double.parseDouble(text));
			} catch (NumberFormatException e1) {
				for (int i = 0; i < validFloatCharecters.length; i++) {
					if (validFloatCharecters[i].equals(text)) {
						result = new Double(0);
						return result;
					}
				}
				throw new IllegalArgumentException(text);
			}
		} else if (c == Float.class || c == Float.TYPE) {
			try {
				result = new Float(Float.parseFloat(text));
			} catch (NumberFormatException e1) {
				for (int i = 0; i < validFloatCharecters.length; i++) {
					if (validFloatCharecters[i].equals(text)) {
						result = new Float(0F);
						return result;
					}
				}
				throw new IllegalArgumentException(text);
			}
		} else if (c == Character.class || c == Character.TYPE) {
			if (text.length() > 1)
				throw new IllegalArgumentException(text);
			else if (text.length() == 0)
				result = null;
			else
				result = new Character(text.charAt(0));
		} else if (c == Boolean.class || c == Boolean.TYPE) {
			// Assume English false for everyting but an exact true
			String val = text.toLowerCase().trim();
			if (val.equals("true"))
				result = Boolean.TRUE;
			else
				result = Boolean.FALSE;
		}
		return result;
	}

	/**
	 * Check to see if the autoEditable feature is enabled.
	 * @return Returns the autoEnable.
	 */
	public boolean isAutoEditable() {
		return autoEditable;
	}

	/**
	 * Set the auto enable property, which automatically enables/disables
	 * editing the text field when the bound data value is valid.
	 * 
	 * @param autoEnable true to enable the autoEditable feature, false to disable.
	 */
	public void setAutoEditable(boolean autoEnable) {
		this.autoEditable = autoEnable;
		if (visualTextComponent != null) {
			visualTextComponent.setEditable(originalEditable);
		}
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		if (fdebugMsg != null) {
			sb.append("***Error**: ");
			sb.append(fdebugMsg);
			sb.append("\n");
		}
		sb.append("SwingTextComponentBinder\n\tProperty: ");
		sb.append(getProperty());
		sb.append("\n");
		return sb.toString();
	}
}